<?php
/* $Id: ce_plugin.class.php,v 1.12 2009/06/29 11:43:18 Criss Exp $ */
if (!defined('PHPWG_ROOT_PATH')) die('Hacking attempt!');

/**
 * CE_Plugin class
 */
class CE_Plugin {

  var $comments_ids;
  var $config;
  var $plugin_id;

  /* ************************ */
  /* ** Constructor        ** */
  /* ************************ */

  function CE_Plugin($plugin_id, $config = null) {
    $this->plugin_id = $plugin_id;
    $this->config = $config;
    if (null != $this->config) {
      $this->config->loadConfig();
    }
  }

  function getConfig() {
    return $this->config;
  }

  /* ************************ */
  /* ** Trigger management ** */
  /* ************************ */

  function get_admin_plugin_menu_links($menu) {
    array_push(
        $menu,
        array(
            'NAME' => $this->plugin_id,
            'URL' => $this->get_plugin_admin_url()
        )
    );
    return $menu;
  }

  /**
   * Update comment ID array
   */
  function render_comment_content($content) {
    $author = null;
    $id = null;
    switch (script_basename()) {
      case "comments":
        global $comment;
        $author = $comment['author'];
        $id = $comment['comment_id'];
        break;
      case "picture":
        global $row;
        $author = $row['author'];
        $id = $row['id'];
        break;
      default:
        // Not in right script
        return $content;
    }
    global $template, $conf, $user;
    if ((!is_a_guest() and ($user[$conf['user_fields']['username']] == $author))
        or is_admin()) {
      $key = count($template->get_template_vars('comments'));
      $this->comments_ids[$key] = $id;
    }
    return $content;
  }

  /**
   * Check whether the page contains comments.
   * Add an edit link if user is allowed to edit comment
   */
  function loc_begin_page_header() {
    global $template;
    $comments = $template->get_template_vars('comments');
    if (!isset($comments) or (null == $comments)) {
      // No comment...
      return;
    }
    // Include language advices
    load_language('plugin.lang', CE_PATH);
    foreach ($comments as $key => $value) {
      if (isset($this->comments_ids[$key])) {
        // User allowed to have edit link
        $current_id = $this->comments_ids[$key];
        $comments[$key]['U_ID'] =  $current_id;
        $comments[$key]['DATE'] .= ' - <a href="';
        $comments[$key]['DATE'] .= $this->getEditUrl($current_id);
        $comments[$key]['DATE'] .= '">';
        $comments[$key]['DATE'] .= l10n('comment_edit_tool');
        $comments[$key]['DATE'] .= '</a>';
      }
    }
    $template->assign('comments', $comments);
  }

  /**
   * Check whether a comment edit or update is requested.
   */
  function loc_begin_index() {
    $this->doAction();
  }

  function loc_begin_picture() {
    $this->doAction();
  }

  /* ************************ */
  /* ** Accessors          ** */
  /* ************************ */

  function getVersion() {
    if (isset($this->config->config_values[CE_CFG_VERSION])) {
      return $this->config->config_values[CE_CFG_VERSION];
    }
    return CE_VERSION;
  }

  /* ************************ */
  /* ** Private functions  ** */
  /* ************************ */

  function doAction() {
    $comment_id = "";
    $action = $this->isCommentEdit($_SERVER['QUERY_STRING'], $comment_id);
    switch ($action) {
      case CE_ACTION_EDIT:
        // Edit
        $this->editComment($comment_id);
        break;
      case CE_ACTION_UPDATE:
        $this->updateComment();
        break;
      case CE_ACTION_ERROR:
        $infos = array('comment_edit_forbidden');
        $this->displayError($infos);
        break;
      case CE_ACTION_NONE:
        // Not an edit mode
        break;
    }
  }

  function getTemplate($p_template = 'edit.tpl') {
    return realpath(CE_TEMPLATE . $p_template);
  }

  function isCommentEdit($p_query_string, &$p_comment_id) {
    $params = split('\&', $p_query_string);
    $action = CE_ACTION_NONE;
    $comment_id = "";
    foreach ($params as $current_param) {
      $key_value = split('=', $current_param);
      if (isset($key_value[1])) {
        if (CE_ACTION == $key_value[0] && CE_ACTION_NONE == $action) {
          switch ($key_value[1]) {
            case CE_ACTION_EDIT:
            case CE_ACTION_UPDATE:
              $action = $key_value[1];
              break;
            default:
          }
        }
        if (CE_ID == $key_value[0]) {
          $comment_id = $key_value[1];
        }
      }
    }
    if (CE_ACTION_EDIT == $action) {
      if (is_numeric($comment_id)) {
        $p_comment_id = intval($comment_id);
      } else {
        $action = CE_ACTION_ERROR;
      }
    }
    return $action;
  }

  function isEditAllowed($p_comment) {
    if ((FALSE == $p_comment->getInfo()) or
        is_a_guest()) {
      return false;
    }
    if (is_admin() and !is_adviser()) {
      return true;
    }
    global $conf, $user;
    return $p_comment->isAuthor($user[$conf['user_fields']['username']]);
  }

  function editComment($p_comment_id) {
    $comment = new CE_Comment($p_comment_id);
    $infos = array();
    if (!$this->isEditAllowed($comment)) {
      // Not allowed
      array_push($infos, 'comment_edit_forbidden');
      $this->displayError($infos);
      return;
    }
    $this->displayComment($comment);
  }

  function updateComment() {
    $infos = array();
    if ((!isset($_POST['ce_commentid'])) or
        (!isset($_POST['ce_imageid']))) {
      array_push($infos, 'comment_access_invalid');
      $this->displayError($infos);
      return;
    }
    if (!is_numeric($_POST['ce_imageid'])) {
      array_push($infos, 'comment_access_invalid');
      $this->displayError($infos);
      return;
    }
    $comment = new CE_Comment($_POST['ce_commentid']);

    if (!$this->isEditAllowed($comment)) {
      array_push($infos, 'comment_edit_forbidden');
      $this->displayError($infos);
      return;
    }
    $image_id = $comment->getInfo('image_id');
    if (intval($_POST['ce_imageid']) != intval($image_id)) {
      array_push($infos, 'comment_access_invalid');
      $this->displayError($infos);
      return;
    }
    if (isset($_POST['ce_author'])) {
      $comment->setInfo('author', $_POST['ce_author']);
    } else if ((isset($_POST['ce_freeauthorck'])) and
               (isset($_POST['ce_freeauthor']))) {
      // Free author
      $new_author = trim(stripslashes($_POST['ce_freeauthor']));
      $new_author = $comment->validateAuthor($new_author);
      $comment->setInfo('author', $new_author);
    }

    $comment_array = array(
      'author'      => trim( stripslashes($comment->getInfo('author')) ),
      'content'     => trim( stripslashes($_POST['ce_content']) ),
      'image_id'    => $image_id,
      'comment_id'  => $comment->getInfo('comment_id'),
    );

    // Deactivate anti-flood system
    global $conf;
    $old_anti_flood = $conf['anti-flood_time'];
    $conf['anti-flood_time'] = 0;

    $comment_action = update_user_comment($comment_array, @$_POST['key'], $infos );

    // Reactivate anti-flood system
    $conf['anti-flood_time'] = $old_anti_flood;

    switch ($comment_action) {
      case 'moderate':
        array_push( $infos, 'comment_to_validate' );
      case 'validate':
        array_push( $infos, 'comment_added');
        break;
      case 'reject':
        set_status_header(403);
        array_push($infos, 'comment_not_added' );
        break;
      default:
        trigger_error('Invalid comment action '.$comment_action, E_USER_WARNING);
    }

    // allow plugins to notify what's going on
    trigger_action( 'user_comment_insertion',
                    array_merge($comment_array, array('action'=>$comment_action)));
    if ($comment_action=='reject') {
      $this->displayError($infos);
    } else {
      $this->displayInfo($infos);
    }
  }

  function getCommentId($p_string_delete) {
    $comment_id = "";
    $elements = split('\?', $p_string_delete);
    if (count($elements) > 1) {
      $elem_var = split('\&amp;', $elements[1]);
      foreach ($elem_var as $var) {
        $key_value = split('=', $var);
        if (count($key_value) > 1) {
          switch ($key_value[0]) {
            case 'comment_to_delete':
            case 'delete':
              $comment_id = $key_value[1];
              break;
            default:
              // No ID
          }
        }
      }
    }
    return $comment_id;
  }

  function displayError($p_error) {
    if ($this->config->getValue(CE_CFG_DISPLAY_ERROR)) {
        $this->displayMessage($p_error, CE_TYPE_ERROR, CE_ERRORS);
    }
  }

  function displayInfo($p_info) {
    if ($this->config->getValue(CE_CFG_DISPLAY_INFO)) {
      $this->displayMessage($p_info, CE_TYPE_INFO, CE_INFOS);
    }
  }

  function displayMessage($p_message, $p_type, $p_image) {
    // Include language advices
    load_language('plugin.lang', CE_PATH);

    $messages = array();
    foreach ($p_message as $message) {
      array_push($messages, l10n($message));
    }
    $message_array = array(
      'TYPE'  => $p_type,
      'IMG'   => $p_image,
      'TITLE' => l10n($p_type),
      'MSG'   => $messages
    );

    global $template;
    if ("" == $template->get_template_vars('U_HOME')) {
      $template->assign('U_HOME', make_index_url());
    }
    $template->set_filenames(array('ce_message'=>$this->getTemplate('message.tpl')));
    $template->block_html_head( '',
                  '<link rel="stylesheet" type="text/css" href="'
                  .CE_INCLUDE . $this->getScriptName() . '.css'
                  .'">',
    $smarty, $repeat);
    if (!$this->setBeginEndFields($begin, $end)) {
      return;
    }

    $template->assign('message', $message_array);
    $old_begin = $template->get_template_vars($begin);
    $template->assign($begin, $template->parse('ce_message', true));
    $template->concat($begin, $old_begin);
  }

  function displayComment($p_comment) {
    // Include language advices
    load_language('plugin.lang', CE_PATH);

    global $template;
    $template->set_filenames(array('ce_edit'=>$this->getTemplate()));
    $template->block_html_head( '',
                  '<link rel="stylesheet" type="text/css" href="'
                  .CE_INCLUDE . $this->getScriptName() . '.css'
                  .'">',
    $smarty, $repeat);

    if (!$this->setBeginEndFields($begin, $end)) {
      return;
    }

    $tpl_comment = array(
      'CONTENT'   => $p_comment->getInfo('content'),
      'KEY'       => $p_comment->getInfo('comment_id'),
      'AUTHOR'    => $p_comment->getInfo('author'),
      'IMAGE'     => $p_comment->getInfo('image_id'),
      'DISABLED'  => !is_admin(),
      'USERS'     => CE_Comment::getUsers(),
      'F_ACTION'  => $this->getUpdateUrl(),
      'FREEAUTHOR'=> ($p_comment->isKnownAuthor())?'':'checked="checked"'
      );
      if ("" == $template->get_template_vars('U_HOME')) {
        $template->assign('U_HOME', make_index_url());
      }
      $template->assign('comment', $tpl_comment);
      $old_begin = $template->get_template_vars($begin);
      $template->assign($begin, $template->parse('ce_edit', true));
      $template->concat($begin, $old_begin);
  }

  function setBeginEndFields(&$begin, &$end) {
    switch (script_basename()) {
      case 'index':
        $begin = 'PLUGIN_INDEX_CONTENT_BEFORE';
        $end = 'PLUGIN_INDEX_CONTENT_AFTER';
        break;
      case 'picture':
        $begin = 'PLUGIN_PICTURE_BEFORE';
        $end = 'PLUGIN_PICTURE_AFTER';
        break;
      default:
        return FALSE;
    }
    return TRUE;
  }

  function getScriptName() {
    switch (script_basename()) {
      case 'index':
      case 'picture':
        return script_basename();
      default:
        return 'index';
    }
  }

  function getEditUrl($comment_id) {
    $url  = get_root_url() . $this->getScriptName() . '.php';
    $url .= get_query_string_diff(array(CE_ACTION, CE_ID));
    $url = add_url_params($url,
                          array(
                            CE_ACTION => CE_ACTION_EDIT,
                            CE_ID=>$comment_id
                          )
            );
    return $url;
  }

  function getUpdateUrl() {
    $url  = get_root_url() . $this->getScriptName() .'.php';
    $url .= get_query_string_diff(array(CE_ACTION, CE_ID));
    $url = add_url_params($url,
                          array(
                            CE_ACTION => CE_ACTION_UPDATE
                          )
            );
    return $url;
  }

  function get_plugin_admin_url() {
    return get_admin_plugin_menu_link(CE_PATH . 'administration.php');
  }

}
?>